
// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tWVP: WORLDVIEWPROJECTION;

float4x4 TT ;

float4 col : COLOR <String uiname="Color";>  = {1, 1, 1, 1};

float2 PARAM1;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

float4x4 tColor <string uiname="Color Transform";>;

struct vs2ps
{
    float4 PosWVP    : POSITION;
    float4 TexCd     : TEXCOORD0;
    float4 colpos : TEXCOORD1;
};

// --------------------------------------------------------------------------------------------------
// FUNCTIONS:
// --------------------------------------------------------------------------------------------------

#define TWOPI 6.28318531
#define PI    3.14159265

float3 sphere(float u, float v)
 {
    float x =  cos(v) * sin(u) ;
    float y =  sin(v) * sin(u) ;
    float z =  cos(u) ;

    return float3(x, y, z);
}


// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS_sphere(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;
    PosO =  mul(PosO, TT);

    float x, y, z, u, v;

    u = (PosO.x) * TWOPI;
    v = (PosO.y) * PI;

    //get position
    PosO.xyz =sphere(u, v);
    PosO.w = 1;
    
    //transform position
    float4 calc = pow(abs(PosO)*2,2);
    PosO.xy=lerp(PosO.xy,calc.yx*PosO.xy,PARAM1);

    
    Out.colpos = length(PosO);

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd  = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS_sphere(vs2ps In): COLOR
{
       float4 src = tex2D(Samp, In.TexCd);
       float4 col1 = (In.colpos * src * col);
       col1.a = 1 ;
       
    return mul(col1,tColor);
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------
technique sphere
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS_sphere();
        PixelShader = compile ps_2_0 PS_sphere();
    }
}
